Class: DataMapper::Validations::GenericValidator

Inherits:
Object
  • Object
show all
Defined in:
lib/dm-validations/validators/generic_validator.rb

Overview

All validators extend this base class. Validators must:

  • Implement the initialize method to capture its parameters, also calling super to have this parent class capture the optional, general :if and :unless parameters.

  • Implement the call method, returning true or false. The call method provides the validation logic.

Author:

  • Guy van den Berg

Since:

  • 0.9

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(field_name, options = {}) ⇒ GenericValidator

Construct a validator. Capture the :if and :unless clauses when present.

All additional key/value pairs are passed through to the validator that is sub-classing this GenericValidator

Parameters:

  • field (String, Symbol)

    The property specified for validation

  • :if<Symbol, (Hash)

    a customizable set of options

  • :unless<Symbol, (Hash)

    a customizable set of options

Since:

  • 0.9



31
32
33
34
35
36
37
# File 'lib/dm-validations/validators/generic_validator.rb', line 31

def initialize(field_name, options = {})
  @field_name           = field_name
  @options              = DataMapper::Ext::Hash.except(options, :if, :unless)
  @if_clause            = options[:if]
  @unless_clause        = options[:unless]
  @humanized_field_name = DataMapper::Inflector.humanize(@field_name)
end

Instance Attribute Details

#field_nameObject (readonly)

Since:

  • 0.9



18
19
20
# File 'lib/dm-validations/validators/generic_validator.rb', line 18

def field_name
  @field_name
end

#humanized_field_nameObject (readonly)

Since:

  • 0.9



18
19
20
# File 'lib/dm-validations/validators/generic_validator.rb', line 18

def humanized_field_name
  @humanized_field_name
end

#if_clauseObject

Since:

  • 0.9



17
18
19
# File 'lib/dm-validations/validators/generic_validator.rb', line 17

def if_clause
  @if_clause
end

#optionsObject (readonly)

Since:

  • 0.9



18
19
20
# File 'lib/dm-validations/validators/generic_validator.rb', line 18

def options
  @options
end

#unless_clauseObject

Since:

  • 0.9



17
18
19
# File 'lib/dm-validations/validators/generic_validator.rb', line 17

def unless_clause
  @unless_clause
end

Instance Method Details

#==(other) ⇒ Object

Returns true if validators are equal

Note that this intentionally do validate options equality

even though it is hard to imagine a situation when multiple validations will be used on the same field with the same conditions but different options, it happens to be the case every once in a while with inferred validations for strings/text and explicitly given validations with different option (usually as Range vs. max limit for inferred validation)

Since:

  • 0.9



122
123
124
125
126
127
128
# File 'lib/dm-validations/validators/generic_validator.rb', line 122

def ==(other)
  self.class == other.class &&
  self.field_name == other.field_name &&
  self.if_clause == other.if_clause &&
  self.unless_clause == other.unless_clause &&
  self.instance_variable_get(:@options) == other.instance_variable_get(:@options)
end

#add_error(target, message, field_name = :general) ⇒ Object

Add an error message to a target resource. If the error corresponds to a specific field of the resource, add it to that field, otherwise add it as a :general message.

TODO - should the field_name for a general message be :default???

Parameters:

  • target (Object)

    the resource that has the error

  • message (String)

    the message to add

  • field_name (Symbol) (defaults to: :general)

    the name of the field that caused the error

Since:

  • 0.9



49
50
51
# File 'lib/dm-validations/validators/generic_validator.rb', line 49

def add_error(target, message, field_name = :general)
  target.errors.add(field_name, message)
end

#call(target) ⇒ Boolean

Call the validator. “call” is used so the operation is BoundMethod and Block compatible. This must be implemented in all concrete classes.

Parameters:

  • target (Object)

    the resource that the validator must be called against

Returns:

  • (Boolean)

    true if valid, otherwise false

Raises:

  • (NotImplementedError)

Since:

  • 0.9



59
60
61
# File 'lib/dm-validations/validators/generic_validator.rb', line 59

def call(target)
  raise NotImplementedError, "#{self.class}#call must be implemented"
end

#execute?(target) ⇒ Boolean

Determines if this validator should be run against the target by evaluating the :if and :unless clauses optionally passed while specifying any validator.

Parameters:

  • target (Object)

    the resource that we check against

Returns:

  • (Boolean)

    true if should be run, otherwise false

Since:

  • 0.9



69
70
71
72
73
74
75
76
77
78
79
80
81
# File 'lib/dm-validations/validators/generic_validator.rb', line 69

def execute?(target)
  if unless_clause = self.unless_clause
    return !target.__send__(unless_clause) if unless_clause.kind_of?(Symbol)
    return !unless_clause.call(target)     if unless_clause.respond_to?(:call)
  end

  if if_clause = self.if_clause
    return target.__send__(if_clause) if if_clause.kind_of?(Symbol)
    return if_clause.call(target)     if if_clause.respond_to?(:call)
  end

  true
end

#inspectObject Also known as: to_s

Since:

  • 0.9



130
131
132
# File 'lib/dm-validations/validators/generic_validator.rb', line 130

def inspect
  "<##{self.class.name} @field_name='#{@field_name}' @if_clause=#{@if_clause.inspect} @unless_clause=#{@unless_clause.inspect} @options=#{@options.inspect}>"
end

#optional?(value) ⇒ Boolean

Test the value to see if it is blank or nil, and if it is allowed. Note that allowing blank without explicitly denying nil allows nil values, since nil.blank? is true.

Parameters:

Returns:

  • (Boolean)

    true if blank/nil is allowed, and the value is blank/nil

Since:

  • 0.9



99
100
101
102
103
104
105
# File 'lib/dm-validations/validators/generic_validator.rb', line 99

def optional?(value)
  if value.nil?
    @options[:allow_nil] || (@options[:allow_blank] && !@options.has_key?(:allow_nil))
  elsif DataMapper::Ext.blank?(value)
    @options[:allow_blank]
  end
end

#set_optional_by_default(default = true) ⇒ undefined

Set the default value for allow_nil and allow_blank

Parameters:

  • default (Boolean) (defaults to: true)

    value

Returns:

  • (undefined)

Since:

  • 0.9



87
88
89
90
91
# File 'lib/dm-validations/validators/generic_validator.rb', line 87

def set_optional_by_default(default = true)
  [ :allow_nil, :allow_blank ].each do |key|
    @options[key] = true unless options.key?(key)
  end
end