Module: DataMapper::Validations::ValidatesPrimitiveType
- Extended by:
- Deprecate
- Included in:
- ClassMethods
- Defined in:
- lib/dm-validations/validators/primitive_validator.rb
Overview
class PrimitiveTypeValidator
Instance Method Summary (collapse)
-
- (Object) validates_primitive_type_of(*fields)
Validates that the specified attribute is of the correct primitive type.
Instance Method Details
- (Object) validates_primitive_type_of(*fields)
Validates that the specified attribute is of the correct primitive type.
51 52 53 |
# File 'lib/dm-validations/validators/primitive_validator.rb', line 51 def validates_primitive_type_of(*fields) validators.add(PrimitiveTypeValidator, *fields) end |